@@ -1,9 +1,10 @@
 /*
- * Copyright  2002-2004 The Apache Software Foundation
- *
- *  Licensed under the Apache License, Version 2.0 (the "License");
- *  you may not use this file except in compliance with the License.
- *  You may obtain a copy of the License at
+ *  Licensed to the Apache Software Foundation (ASF) under one or more
+ *  contributor license agreements.  See the NOTICE file distributed with
+ *  this work for additional information regarding copyright ownership.
+ *  The ASF licenses this file to You under the Apache License, Version 2.0
+ *  (the "License"); you may not use this file except in compliance with
+ *  the License.  You may obtain a copy of the License at
  *
  *      http://www.apache.org/licenses/LICENSE-2.0
  *
@@ -74,6 +75,9 @@ public class PropertyHelper {
     protected PropertyHelper() {
     }
 
+    //override facility for subclasses to put custom hashtables in
+
+
     // --------------------  Hook management  --------------------
 
     /**
@@ -121,14 +125,14 @@ public class PropertyHelper {
     public static synchronized
         PropertyHelper getPropertyHelper(Project project) {
         PropertyHelper helper
-            = (PropertyHelper) project.getReference("ant.PropertyHelper");
+            = (PropertyHelper) project.getReference(MagicNames.REFID_PROPERTY_HELPER);
         if (helper != null) {
             return helper;
         }
         helper = new PropertyHelper();
         helper.setProject(project);
 
-        project.addReference("ant.PropertyHelper", helper);
+        project.addReference(MagicNames.REFID_PROPERTY_HELPER, helper);
         return helper;
     }
 
@@ -167,7 +171,6 @@ public class PropertyHelper {
                 return true;
             }
         }
-
         return false;
     }
 
@@ -190,13 +193,8 @@ public class PropertyHelper {
         if (name.startsWith("toString:")) {
             name = name.substring("toString:".length());
             Object v = project.getReference(name);
-            if (v == null) {
-                return null;
-            }
-            return v.toString();
+            return (v == null) ? null : v.toString();
         }
-
-
         return null;
     }
 
@@ -249,13 +247,11 @@ public class PropertyHelper {
      * @return the original string with the properties replaced, or
      *         <code>null</code> if the original string is <code>null</code>.
      */
-    public String replaceProperties(String ns, String value,
-                                    Hashtable keys)
+    public String replaceProperties(String ns, String value, Hashtable keys)
             throws BuildException {
-        if (value == null) {
-            return null;
+        if (value == null || value.indexOf('$') == -1) {
+            return value;
         }
-
         Vector fragments = new Vector();
         Vector propertyRefs = new Vector();
         parsePropertyString(value, fragments, propertyRefs);
@@ -280,8 +276,8 @@ public class PropertyHelper {
                 }
 
                 if (replacement == null) {
-                    project.log("Property ${" + propertyName
-                            + "} has not been set", Project.MSG_VERBOSE);
+                    project.log("Property \"" + propertyName
+                            + "\" has not been set", Project.MSG_VERBOSE);
                 }
                 fragment = (replacement != null)
                         ? replacement.toString()
@@ -289,7 +285,6 @@ public class PropertyHelper {
             }
             sb.append(fragment);
         }
-
         return sb.toString();
     }
 
@@ -312,8 +307,8 @@ public class PropertyHelper {
         // user (CLI) properties take precedence
         if (null != userProperties.get(name)) {
             if (verbose) {
-                project.log("Override ignored for user property " + name,
-                        Project.MSG_VERBOSE);
+                project.log("Override ignored for user property \"" + name
+                    + "\"", Project.MSG_VERBOSE);
             }
             return false;
         }
@@ -324,8 +319,8 @@ public class PropertyHelper {
         }
 
         if (null != properties.get(name) && verbose) {
-            project.log("Overriding previous definition of property " + name,
-                    Project.MSG_VERBOSE);
+            project.log("Overriding previous definition of property \"" + name
+                + "\"", Project.MSG_VERBOSE);
         }
 
         if (verbose) {
@@ -351,8 +346,8 @@ public class PropertyHelper {
     public synchronized void setNewProperty(String ns, String name,
                                             Object value) {
         if (null != properties.get(name)) {
-            project.log("Override ignored for property " + name,
-                    Project.MSG_VERBOSE);
+            project.log("Override ignored for property \"" + name
+                + "\"", Project.MSG_VERBOSE);
             return;
         }
 
@@ -469,8 +464,6 @@ public class PropertyHelper {
     // deprecated, it is possible to use a better (more efficient)
     // mechanism to preserve the context.
 
-    // TODO: do we need to delegate ?
-
     /**
      * Returns a copy of the properties table.
      * @return a hashtable containing all properties
@@ -490,6 +483,36 @@ public class PropertyHelper {
         return new Hashtable(userProperties);
     }
 
+    /**
+     * special back door for subclasses, internal access to
+     * the hashtables
+     * @return the live hashtable of all properties
+     */
+    protected Hashtable getInternalProperties() {
+        return properties;
+    }
+
+    /**
+     * special back door for subclasses, internal access to
+     * the hashtables
+     *
+     * @return the live hashtable of user properties
+     */
+    protected Hashtable getInternalUserProperties() {
+        return userProperties;
+    }
+
+    /**
+     * special back door for subclasses, internal access to
+     * the hashtables
+     *
+     * @return the live hashtable inherited properties
+     */
+    protected Hashtable getInternalInheritedProperties() {
+        return inheritedProperties;
+    }
+
+
     /**
      * Copies all user properties that have not been set on the
      * command line or a GUI tool from this instance to the Project
